@@ -4,7 +4,9 @@ import warnings
 from collections.abc import Sequence
 
 import numpy as np
+import packaging
 import pandas as pd
+import scipy
 from scipy import stats
 
 from ..data import convert_to_dataset
@@ -25,15 +27,16 @@ def bfmi(data):
 
     BFMI quantifies how well momentum resampling matches the marginal energy distribution. For more
     information on BFMI, see https://arxiv.org/pdf/1604.00695v1.pdf. The current advice is that
-    values smaller than 0.3 indicate poor sampling. However, this threshold is provisional and may
-    change. See http://mc-stan.org/users/documentation/case-studies/pystan_workflow.html for more
-    information.
+    values smaller than 0.3 indicate poor sampling. However, this threshold is
+    provisional and may change. See
+    `pystan_workflow <http://mc-stan.org/users/documentation/case-studies/pystan_workflow.html>`_
+    for more information.
 
     Parameters
     ----------
     data : obj
-        Any object that can be converted to an az.InferenceData object.
-        Refer to documentation of az.convert_to_dataset for details.
+        Any object that can be converted to an :class:`arviz.InferenceData` object.
+        Refer to documentation of :func:`arviz.convert_to_dataset` for details.
         If InferenceData, energy variable needs to be found.
 
     Returns
@@ -42,6 +45,11 @@ def bfmi(data):
         The Bayesian fraction of missing information of the model and trace. One element per
         chain in the trace.
 
+    See Also
+    --------
+    plot_energy : Plot energy transition distribution and marginal energy
+                  distribution in HMC algorithms.
+
     Examples
     --------
     Compute the BFMI of an InferenceData object
@@ -59,7 +67,7 @@ def bfmi(data):
     dataset = convert_to_dataset(data, group="sample_stats")
     if not hasattr(dataset, "energy"):
         raise TypeError("Energy variable was not found.")
-    return _bfmi(dataset.energy)
+    return _bfmi(dataset.energy.transpose("chain", "draw"))
 
 
 def ess(
@@ -76,10 +84,10 @@ def ess(
     Parameters
     ----------
     data : obj
-        Any object that can be converted to an ``az.InferenceData`` object.
-        Refer to documentation of ``az.convert_to_dataset`` for details.
+        Any object that can be converted to an :class:`arviz.InferenceData` object.
+        Refer to documentation of :func:`arviz.convert_to_dataset` for details.
         For ndarray: shape = (chain, draw).
-        For n-dimensional ndarray transform first to dataset with ``az.convert_to_dataset``.
+        For n-dimensional ndarray transform first to dataset with :func:`arviz.convert_to_dataset`.
     var_names : str or list of str
         Names of variables to include in the return value Dataset.
     method : str, optional, default "bulk"
@@ -98,7 +106,7 @@ def ess(
         - "local"
     relative : bool
         Return relative ess
-        `ress = ess / n`
+        ``ress = ess / n``
     prob : float, or tuple of two floats, optional
         probability value for "tail", "quantile" or "local" ess functions.
     dask_kwargs : dict, optional
@@ -127,10 +135,18 @@ def ess(
 
     References
     ----------
-    * Vehtari et al. (2019) see https://arxiv.org/abs/1903.08008
-    * https://mc-stan.org/docs/2_18/reference-manual/effective-sample-size-section.html
-      Section 15.4.2
-    * Gelman et al. BDA (2014) Formula 11.8
+    * Vehtari et al. (2021). Rank-normalization, folding, and
+        localization: An improved Rhat for assessing convergence of
+        MCMC. Bayesian analysis, 16(2):667-718.
+    * https://mc-stan.org/docs/reference-manual/analysis.html#effective-sample-size.section
+    * Gelman et al. BDA3 (2013) Formula 11.8
+
+    See Also
+    --------
+    arviz.rhat : Compute estimate of rank normalized splitR-hat for a set of traces.
+    arviz.mcse : Calculate Markov Chain Standard Error statistic.
+    plot_ess : Plot quantile, local or evolution of effective sample sizes (ESS).
+    arviz.summary : Create a data frame with summary statistics.
 
     Examples
     --------
@@ -184,14 +200,14 @@ def ess(
                 return ess_func(  # pylint: disable=unexpected-keyword-arg
                     data, prob=prob, relative=relative
                 )
-            else:
-                return ess_func(data, relative=relative)
-        else:
-            msg = (
-                "Only uni-dimensional ndarray variables are supported."
-                " Please transform first to dataset with `az.convert_to_dataset`."
-            )
-            raise TypeError(msg)
+
+            return ess_func(data, relative=relative)
+
+        msg = (
+            "Only uni-dimensional ndarray variables are supported."
+            " Please transform first to dataset with `az.convert_to_dataset`."
+        )
+        raise TypeError(msg)
 
     dataset = convert_to_dataset(data, group="posterior")
     var_names = _var_names(var_names, dataset)
@@ -221,17 +237,17 @@ def rhat(data, *, var_names=None, method="rank", dask_kwargs=None):
     Parameters
     ----------
     data : obj
-        Any object that can be converted to an az.InferenceData object.
-        Refer to documentation of az.convert_to_dataset for details.
+        Any object that can be converted to an :class:`arviz.InferenceData` object.
+        Refer to documentation of :func:`arviz.convert_to_dataset` for details.
         At least 2 posterior chains are needed to compute this diagnostic of one or more
         stochastic parameters.
         For ndarray: shape = (chain, draw).
-        For n-dimensional ndarray transform first to dataset with az.convert_to_dataset.
+        For n-dimensional ndarray transform first to dataset with ``az.convert_to_dataset``.
     var_names : list
         Names of variables to include in the rhat report
     method : str
         Select R-hat method. Valid methods are:
-        - "rank"        # recommended by Vehtari et al. (2019)
+        - "rank"        # recommended by Vehtari et al. (2021)
         - "split"
         - "folded"
         - "z_scale"
@@ -244,24 +260,33 @@ def rhat(data, *, var_names=None, method="rank", dask_kwargs=None):
     xarray.Dataset
       Returns dataset of the potential scale reduction factors, :math:`\hat{R}`
 
+    See Also
+    --------
+    ess : Calculate estimate of the effective sample size (ess).
+    mcse : Calculate Markov Chain Standard Error statistic.
+    plot_forest : Forest plot to compare HDI intervals from a number of distributions.
+
     Notes
     -----
     The diagnostic is computed by:
 
-      .. math:: \hat{R} = \frac{\hat{V}}{W}
+      .. math:: \hat{R} = \sqrt{\frac{\hat{V}}{W}}
 
     where :math:`W` is the within-chain variance and :math:`\hat{V}` is the posterior variance
     estimate for the pooled rank-traces. This is the potential scale reduction factor, which
     converges to unity when each of the traces is a sample from the target posterior. Values
     greater than one indicate that one or more chains have not yet converged.
 
-    Rank values are calculated over all the chains with `scipy.stats.rankdata`.
-    Each chain is split in two and normalized with the z-transform following Vehtari et al. (2019).
+    Rank values are calculated over all the chains with ``scipy.stats.rankdata``.
+    Each chain is split in two and normalized with the z-transform following
+    Vehtari et al. (2021).
 
     References
     ----------
-    * Vehtari et al. (2019) see https://arxiv.org/abs/1903.08008
-    * Gelman et al. BDA (2014)
+    * Vehtari et al. (2021).  Rank-normalization, folding, and
+      localization: An improved Rhat for assessing convergence of
+      MCMC. Bayesian analysis, 16(2):667-718.
+    * Gelman et al. BDA3 (2013)
     * Brooks and Gelman (1998)
     * Gelman and Rubin (1992)
 
@@ -299,12 +324,12 @@ def rhat(data, *, var_names=None, method="rank", dask_kwargs=None):
         data = np.atleast_2d(data)
         if len(data.shape) < 3:
             return rhat_func(data)
-        else:
-            msg = (
-                "Only uni-dimensional ndarray variables are supported."
-                " Please transform first to dataset with `az.convert_to_dataset`."
-            )
-            raise TypeError(msg)
+
+        msg = (
+            "Only uni-dimensional ndarray variables are supported."
+            " Please transform first to dataset with `az.convert_to_dataset`."
+        )
+        raise TypeError(msg)
 
     dataset = convert_to_dataset(data, group="posterior")
     var_names = _var_names(var_names, dataset)
@@ -328,10 +353,10 @@ def mcse(data, *, var_names=None, method="mean", prob=None, dask_kwargs=None):
     Parameters
     ----------
     data : obj
-        Any object that can be converted to an az.InferenceData object
-        Refer to documentation of az.convert_to_dataset for details
+        Any object that can be converted to an :class:`arviz.InferenceData` object
+        Refer to documentation of :func:`arviz.convert_to_dataset` for details
         For ndarray: shape = (chain, draw).
-        For n-dimensional ndarray transform first to dataset with az.convert_to_dataset.
+        For n-dimensional ndarray transform first to dataset with ``az.convert_to_dataset``.
     var_names : list
         Names of variables to include in the rhat report
     method : str
@@ -351,6 +376,12 @@ def mcse(data, *, var_names=None, method="mean", prob=None, dask_kwargs=None):
     xarray.Dataset
         Return the msce dataset
 
+    See Also
+    --------
+    ess : Compute autocovariance estimates for every lag for the input array.
+    summary : Create a data frame with summary statistics.
+    plot_mcse : Plot quantile or local Monte Carlo Standard Error.
+
     Examples
     --------
     Calculate the Markov Chain Standard Error using the default arguments:
@@ -390,14 +421,14 @@ def mcse(data, *, var_names=None, method="mean", prob=None, dask_kwargs=None):
         if len(data.shape) < 3:
             if prob is not None:
                 return mcse_func(data, prob=prob)  # pylint: disable=unexpected-keyword-arg
-            else:
-                return mcse_func(data)
-        else:
-            msg = (
-                "Only uni-dimensional ndarray variables are supported."
-                " Please transform first to dataset with `az.convert_to_dataset`."
-            )
-            raise TypeError(msg)
+
+            return mcse_func(data)
+
+        msg = (
+            "Only uni-dimensional ndarray variables are supported."
+            " Please transform first to dataset with `az.convert_to_dataset`."
+        )
+        raise TypeError(msg)
 
     dataset = convert_to_dataset(data, group="posterior")
     var_names = _var_names(var_names, dataset)
@@ -430,10 +461,10 @@ def ks_summary(pareto_tail_indices):
     """
     _numba_flag = Numba.numba_flag
     if _numba_flag:
-        bins = np.asarray([-np.Inf, 0.5, 0.7, 1, np.Inf])
+        bins = np.asarray([-np.inf, 0.5, 0.7, 1, np.inf])
         kcounts, *_ = _histogram(pareto_tail_indices, bins)
     else:
-        kcounts, *_ = _histogram(pareto_tail_indices, bins=[-np.Inf, 0.5, 0.7, 1, np.Inf])
+        kcounts, *_ = _histogram(pareto_tail_indices, bins=[-np.inf, 0.5, 0.7, 1, np.inf])
     kprop = kcounts / len(pareto_tail_indices) * 100
     df_k = pd.DataFrame(
         dict(_=["(good)", "(ok)", "(bad)", "(very bad)"], Count=kcounts, Pct=kprop)
@@ -513,7 +544,13 @@ def _z_scale(ary):
     np.ndarray
     """
     ary = np.asarray(ary)
-    rank = stats.rankdata(ary, method="average")
+    if packaging.version.parse(scipy.__version__) < packaging.version.parse("1.10.0.dev0"):
+        rank = stats.rankdata(ary, method="average")
+    else:
+        # the .ravel part is only needed to overcom a bug in scipy 1.10.0.rc1
+        rank = stats.rankdata(  # pylint: disable=unexpected-keyword-arg
+            ary, method="average", nan_policy="omit"
+        )
     rank = _backtransform_ranks(rank)
     z = stats.norm.ppf(rank)
     z = z.reshape(ary.shape)
@@ -523,11 +560,9 @@ def _z_scale(ary):
 def _split_chains(ary):
     """Split and stack chains."""
     ary = np.asarray(ary)
-    if len(ary.shape) > 1:
-        _, n_draw = ary.shape
-    else:
+    if len(ary.shape) <= 1:
         ary = np.atleast_2d(ary)
-        _, n_draw = ary.shape
+    _, n_draw = ary.shape
     half = n_draw // 2
     return _stack(ary[:, :half], ary[:, -half:])
 
@@ -709,8 +744,8 @@ def _ess_sd(ary, relative=False):
     ary = np.asarray(ary)
     if _not_valid(ary, shape_kwargs=dict(min_draws=4, min_chains=1)):
         return np.nan
-    ary = _split_chains(ary)
-    return min(_ess(ary, relative=relative), _ess(ary ** 2, relative=relative))
+    ary = (ary - ary.mean()) ** 2
+    return _ess(_split_chains(ary), relative=relative)
 
 
 def _ess_quantile(ary, prob, relative=False):
@@ -803,13 +838,15 @@ def _mcse_sd(ary):
     ary = np.asarray(ary)
     if _not_valid(ary, shape_kwargs=dict(min_draws=4, min_chains=1)):
         return np.nan
-    ess = _ess_sd(ary)
+    sims_c2 = (ary - ary.mean()) ** 2
+    ess = _ess_mean(sims_c2)
+    evar = (sims_c2).mean()
+    varvar = ((sims_c2**2).mean() - evar**2) / ess
+    varsd = varvar / evar / 4
     if _numba_flag:
-        sd = np.float(_sqrt(svar(np.ravel(ary), ddof=1), np.zeros(1)))
+        mcse_sd_value = float(_sqrt(np.ravel(varsd), np.zeros(1)))
     else:
-        sd = np.std(ary, ddof=1)
-    fac_mcse_sd = np.sqrt(np.exp(1) * (1 - 1 / ess) ** (ess - 1) - 1)
-    mcse_sd_value = sd * fac_mcse_sd
+        mcse_sd_value = np.sqrt(varsd)
     return mcse_sd_value
 
 
@@ -858,7 +895,6 @@ def _mc_error(ary, batches=5, circular=False):
     """
     _numba_flag = Numba.numba_flag
     if ary.ndim > 1:
-
         dims = np.shape(ary)
         trace = np.transpose([t.ravel() for t in ary])
 
@@ -873,11 +909,10 @@ def _mc_error(ary, batches=5, circular=False):
                     std = _circular_standard_deviation(ary, high=np.pi, low=-np.pi)
                 else:
                     std = stats.circstd(ary, high=np.pi, low=-np.pi)
+            elif _numba_flag:
+                std = float(_sqrt(svar(ary), np.zeros(1)).item())
             else:
-                if _numba_flag:
-                    std = np.float(_sqrt(svar(ary), np.zeros(1)))
-                else:
-                    std = np.std(ary)
+                std = np.std(ary)
             return std / np.sqrt(len(ary))
 
         batched_traces = np.resize(ary, (batches, int(len(ary) / batches)))
@@ -890,39 +925,34 @@ def _mc_error(ary, batches=5, circular=False):
                 std = stats.circstd(means, high=np.pi, low=-np.pi)
         else:
             means = np.mean(batched_traces, 1)
-            if _numba_flag:
-                std = _sqrt(svar(means), np.zeros(1))
-            else:
-                std = np.std(means)
-
+            std = _sqrt(svar(means), np.zeros(1)) if _numba_flag else np.std(means)
         return std / np.sqrt(batches)
 
 
-def _multichain_statistics(ary):
+def _multichain_statistics(ary, focus="mean"):
     """Calculate efficiently multichain statistics for summary.
 
     Parameters
     ----------
     ary : numpy.ndarray
+    focus : select focus for the statistics. Deafault is mean.
 
     Returns
     -------
     tuple
         Order of return parameters is
-            - mcse_mean, mcse_sd, ess_mean, ess_sd, ess_bulk, ess_tail, r_hat
+            If focus equals "mean"
+                - mcse_mean, mcse_sd, ess_bulk, ess_tail, r_hat
+            Else if focus equals "median"
+                - mcse_median, ess_median, ess_tail, r_hat
     """
     ary = np.atleast_2d(ary)
     if _not_valid(ary, shape_kwargs=dict(min_draws=4, min_chains=1)):
-        return np.nan, np.nan, np.nan, np.nan, np.nan
-    # ess mean
-    ess_mean_value = _ess_mean(ary)
+        if focus == "mean":
+            return np.nan, np.nan, np.nan, np.nan, np.nan
+        return np.nan, np.nan, np.nan, np.nan
 
-    # ess sd
-    ess_sd_value = _ess_sd(ary)
-
-    # ess bulk
     z_split = _z_scale(_split_chains(ary))
-    ess_bulk_value = _ess(z_split)
 
     # ess tail
     quantile05, quantile95 = _quantile(ary, [0.05, 0.95])
@@ -941,18 +971,43 @@ def _multichain_statistics(ary):
         rhat_tail = _rhat(_z_scale(_split_chains(ary_folded)))
         rhat_value = max(rhat_bulk, rhat_tail)
 
-    # mcse_mean
-    sd = np.std(ary, ddof=1)
-    mcse_mean_value = sd / np.sqrt(ess_mean_value)
+    if focus == "mean":
+        # ess mean
+        ess_mean_value = _ess_mean(ary)
 
-    # mcse_sd
-    fac_mcse_sd = np.sqrt(np.exp(1) * (1 - 1 / ess_sd_value) ** (ess_sd_value - 1) - 1)
-    mcse_sd_value = sd * fac_mcse_sd
+        # mcse_mean
+        sims_c2 = (ary - ary.mean()) ** 2
+        sims_c2_sum = sims_c2.sum()
+        var = sims_c2_sum / (sims_c2.size - 1)
+        mcse_mean_value = np.sqrt(var / ess_mean_value)
+
+        # ess bulk
+        ess_bulk_value = _ess(z_split)
+
+        # mcse_sd
+        evar = sims_c2_sum / sims_c2.size
+        ess_mean_sims = _ess_mean(sims_c2)
+        varvar = ((sims_c2**2).mean() - evar**2) / ess_mean_sims
+        varsd = varvar / evar / 4
+        mcse_sd_value = np.sqrt(varsd)
+
+        return (
+            mcse_mean_value,
+            mcse_sd_value,
+            ess_bulk_value,
+            ess_tail_value,
+            rhat_value,
+        )
+
+    # ess median
+    ess_median_value = _ess_median(ary)
+
+    # mcse_median
+    mcse_median_value = _mcse_median(ary)
 
     return (
-        mcse_mean_value,
-        mcse_sd_value,
-        ess_bulk_value,
+        mcse_median_value,
+        ess_median_value,
         ess_tail_value,
         rhat_value,
     )
